/*********************************************************************************
 *
 * Inviwo - Interactive Visualization Workshop
 *
 * Copyright (c) 2016-2019 Inviwo Foundation
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *********************************************************************************/

#ifndef IVW_SKELETONINFO_H
#define IVW_SKELETONINFO_H

#include <modules/morphologyvisualization/morphologyvisualizationmoduledefine.h>
#include <inviwo/core/common/inviwo.h>
#include <modules/morphologyvisualization/datastructures/skeletonvoxels.h>
#include <modules/base/datastructures/kdtree.h>

namespace inviwo {

namespace skeletonutil {

struct SkeletonInfo {
	SkeletonInfo(std::shared_ptr<const Volume> binaryVolume, std::shared_ptr<const Volume> distVolum);
	void fillupVoxelInfo();

	util::IndexMapper3D index_;
	ivec3 dim_;
	double thres_;
	K3DTree<skeletonutil::Voxel, float> tree_;
	std::shared_ptr<const Volume> maskVolume_;
	std::shared_ptr<const Volume> distVolume_;

	skeletonutil::VoxelMap allVoxels_;
	skeletonutil::VoxelMap foregroundVoxels_;
	skeletonutil::VoxelMap backgroundVoxels_;

	//lamdas
	std::function<bool(const ivec3&)> isForeGround_;
	std::function<bool(ivec3)> voxelBoundaryTest_;
	std::function<size_t(ivec3)> voxelMapping_;
	std::function<vec3(ivec3)> voxelgetNormalizedPos_;
	std::function<std::vector<size3_t>(ivec3, double)> voxelNeighbourHood_;
	std::function<std::vector<skeletonutil::Voxel>(ivec3, double)> voxelNeighbourHoodVoxels_;
	bool reComputeDistance_;
	vec2 minMaxDist_;
};


}
}  // namespace

#endif  // IVW_SKELETONINFO_H
