/*********************************************************************************
 *
 * Inviwo - Interactive Visualization Workshop
 *
 * Copyright (c) 2016-2019 Inviwo Foundation
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *********************************************************************************/

#include <modules/morphologyvisualization/morphologyvisualizationmodule.h>
#include <modules/morphologyvisualization/processors/skeletonextraction.h>
#include <modules/morphologyvisualization/processors/trajectoryclustering.h>
#include <modules/morphologyvisualization/processors/localmaximadistance.h>
#include <modules/morphologyvisualization/processors/meshcontainers.h>
#include <modules/morphologyvisualization/processors/maximalabelgenerator.h>
//#include <modules/morphologyvisualization/processors/trajectorypca.h>
#include <modules/morphologyvisualization/processors/maximaedgepairs.h>
#include <modules/morphologyvisualization/processors/maximaplots.h>
#include <modules/morphologyvisualization/processors/networkgraphs/trajectorymassspringtograph.h>
#include <modules/morphologyvisualization/processors/networkgraphs/trajectorygraphtomesh.h>
#include <modules/morphologyvisualization/util/trajectorylabelmetadata.h>
#include <modules/morphologyvisualization/processors/chargepools.h>
#include <modules/morphologyvisualization/processors/networkgraphs/graphtocircularplotmesh.h>
#include <modules/morphologyvisualization/processors/pointrenderercompat.h>
#include <modules/morphologyvisualization/processors/trajectorysimplification.h>
#include <modules/morphologyvisualization/processors/networkgraphs/adjacencymatrixtomesh.h>
#include <modules/morphologyvisualization/processors/networkgraphs/chargepoolvis.h>
#include <modules/morphologyvisualization/processors/networkgraphs/unifytrajectorygraph.h>
#include <modules/morphologyvisualization/processors/linestotube.h>
#include <modules/morphologyvisualization/processors/trajectoryvariabilityplot.h>
#include <modules/morphologyvisualization/processors/variabilityplotutil.h>


namespace inviwo {

MorphologyVisualizationModule::MorphologyVisualizationModule(InviwoApplication* app)
    : InviwoModule(app, "MorphologyVisualization") {
    // Processors
    registerProcessor<SkeletonExtraction>();
    registerProcessor<TrajectoryClustering>();
    registerProcessor<LocalMaximaDistance>();
    registerProcessor<MaximaLabelGenerator>();
    registerProcessor<MeshContainers>();
    // registerProcessor<TrajectoryPCA>();
    registerProcessor<MaximaEdgePairs>();
    registerProcessor<MaximaPlots>();
    registerProcessor<TrajectorymassspringToGraph>();
    registerProcessor<TrajectoryGraphToMesh>();
    registerProcessor<ChargePools>();
    registerProcessor<GraphToCircularPlotMesh>();
    registerProcessor<PointRendererCompat>();
    registerProcessor<TrajectorySimplification>();
    registerProcessor<AdjacencyMatrixToMesh>();
    registerProcessor<ChargePoolVis>();
    registerProcessor<UnifyTrajectoryGraph>();
    registerProcessor<LinesToTube>();
    registerProcessor<TrajectoryVariabilityPlot>();

    registerMetaData(std::make_unique<LabelMetaData>());
}

int MorphologyVisualizationModule::getVersion() const { return 1; }

std::unique_ptr<VersionConverter> MorphologyVisualizationModule::getConverter(int version) const {
    return std::make_unique<Converter>(version);
}

MorphologyVisualizationModule::Converter::Converter(int version) : version_(version) {}

bool MorphologyVisualizationModule::Converter::convert(TxElement* root) {
    auto makerules = []() {
        std::vector<xml::IdentifierReplacement> repl ={
            // LocalMaximaDistance
            { { xml::Kind::processor("org.inviwo.LocalMaximaDistance"),
            xml::Kind::outport("org.inviwo.MeshOutport") },
            "outport",
            "outputMeshWorldCoord" }
        };
        return repl;
    };

    bool res = false;
    switch (version_) {
        case 0: {
            auto repl = makerules();
            res |= xml::changeIdentifiers(root, repl);
            return res;
        }

        default:
            return false;  // No changes
    }
    return true;
}

}  // namespace
