/*********************************************************************************
 *
 * Inviwo - Interactive Visualization Workshop
 *
 * Copyright (c) 2017-2019 Inviwo Foundation
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 *********************************************************************************/

#ifndef IVW_CHARGEPOOLS_H
#define IVW_CHARGEPOOLS_H

#include <modules/morphologyvisualization/morphologyvisualizationmoduledefine.h>
#include <inviwo/core/common/inviwo.h>

#include <modules/gromacs/ports/gromacport.h>

#include <inviwo/core/processors/processor.h>
#include <inviwo/core/properties/ordinalproperty.h>
#include <inviwo/core/ports/datainport.h>
#include <inviwo/core/ports/volumeport.h>
#include <inviwo/core/ports/meshport.h>

namespace inviwo {

/** \docpage{org.inviwo.ChargePools, Charge Pools}
 * ![](org.inviwo.ChargePools.png?classIdentifier=org.inviwo.ChargePools)
 * Processor for computing the charge pools given input trajectories.
 */


/**
 * \class ChargePools
 * \brief processor for computing the charge pools given input trajectories
 */
class IVW_MODULE_MORPHOLOGYVISUALIZATION_API ChargePools : public Processor { 
public:
    ChargePools();
    virtual ~ChargePools() = default;
     
    virtual void process() override;

    virtual const ProcessorInfo getProcessorInfo() const override;
    static const ProcessorInfo processorInfo_;

private:
    static std::pair<std::shared_ptr<Volume>, std::vector<ivec3>> findMaxima(const std::shared_ptr<const Volume>& distVolume);

    std::shared_ptr<Mesh> getMaximaMesh(const std::vector<ivec3>& maximaIndices, const size3_t& volDim);

    GromacsMultiInportFlat gromacsInport_;
    VolumeInport distanceInport1_;

    VolumeOutport maximaVolOutport_;
    MeshOutport meshOutport_;
    
    FloatVec4Property colorMaxima_;
    FloatVec4Property colorMaximaInterface_;
};

} // namespace inviwo

#endif // IVW_CHARGEPOOLS_H
