/*********************************************************************************
 *
 * Inviwo - Interactive Visualization Workshop
 *
 * Copyright (c) 2016-2019 Inviwo Foundation
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 *********************************************************************************/

#ifndef IVW_LOCALMAXIMADISTANCE_H
#define IVW_LOCALMAXIMADISTANCE_H

#include <modules/morphologyvisualization/morphologyvisualizationmoduledefine.h>
#include <inviwo/core/common/inviwo.h>
#include <inviwo/core/processors/processor.h>
#include <inviwo/core/properties/ordinalproperty.h>
#include <inviwo/core/ports/imageport.h>
#include <modules/gromacs/ports/gromacport.h>
#include <modules/morphologyvisualization/datastructures/skeletonvoxels.h>
#include <modules/base/datastructures/kdtree.h>
#include <modules/morphologyvisualization/datastructures/skeletoninfo.h>
#include <inviwo/core/ports/volumeport.h>
#include <inviwo/core/ports/meshport.h>

namespace inviwo {

/** \docpage{org.inviwo.localmaximadistance, localmaximadistance}
 * ![](org.inviwo.localmaximadistance.png?classIdentifier=org.inviwo.localmaximadistance)
 * Explanation of how to use the processor.
 *
 * ### Inports
 *   * __<Inport1>__ <description>.
 *
 * ### Outports
 *   * __<Outport1>__ <description>.
 * 
 * ### Properties
 *   * __<Prop1>__ <description>.
 *   * __<Prop2>__ <description>
 */


/**
 * \class LocalMaximaDistance
 * \brief VERY_BRIEFLY_DESCRIBE_THE_CLASS 
 * DESCRIBE_THE_CLASS_FROM_A_DEVELOPER_PERSPECTIVE
 */
class IVW_MODULE_MORPHOLOGYVISUALIZATION_API LocalMaximaDistance : public Processor { 
public:
    LocalMaximaDistance();
    virtual ~LocalMaximaDistance() = default;
     
    virtual void process() override;

    virtual const ProcessorInfo getProcessorInfo() const override;
    static const ProcessorInfo processorInfo_;

    void generateVoxelMesh();
    void buildTree();

private:
    void compute3DMaxima();
    void computeInterfaceMaxima();

    void checkForMaxima(const ivec3& centerIndex, const VolumeRAM *volume);

    void checkFor2DMaxima(int zSlice, const ivec3& centerIndex, const VolumeRAM *volume);

    GromacsMultiInportFlat gromacsInport_;
    VolumeInport binaryVolume_;
    VolumeInport distanceVolume_;
    //MeshOutport meshWorldOutport_;
    MeshOutport meshIndexOutport_;

    void updateVolumeProperties();

    BoolProperty enableMaximaComputation_;
    BoolProperty reComputeDistanceTransform_;
    BoolProperty normalizePositions_;
    FloatProperty distanceScaling_;
    IntMinMaxProperty clipZ_;

    FloatVec4Property colorMaxima_;
    FloatVec4Property colorMaximaInterface_;

    std::shared_ptr<BasicMesh> meshIndexCoord_;
    std::shared_ptr<skeletonutil::SkeletonInfo> ski_;

    FloatProperty maximaRadius_;
    K3DTree<std::size_t, float> maximaTree_;
};

} // namespace

#endif // IVW_LOCALMAXIMADISTANCE_H

