/*********************************************************************************
 *
 * Inviwo - Interactive Visualization Workshop
 *
 * Copyright (c) 2016-2019 Inviwo Foundation
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 *********************************************************************************/

#ifndef IVW_MAXIMAEDGEPAIRS_H
#define IVW_MAXIMAEDGEPAIRS_H

#include <modules/morphologyvisualization/morphologyvisualizationmoduledefine.h>
#include <inviwo/core/common/inviwo.h>
#include <inviwo/core/processors/processor.h>
#include <inviwo/core/properties/ordinalproperty.h>
#include <inviwo/core/ports/meshport.h>
#include <modules/morphologyvisualization/util/trajectoryspringmass.h>
#include <inviwo/dataframe/datastructures/dataframe.h>
#include <inviwo/core/properties/transferfunctionproperty.h>
#include <modules/gromacs/properties/dofpropertyset.h>
#include <inviwo/core/properties/compositeproperty.h>
#include <inviwo/core/properties/optionproperty.h>
#include <inviwo/core/properties/cameraproperty.h>
#include <modules/dataplotting/datastructures/dataplottingcommontypes.h>

namespace inviwo {


class IVW_MODULE_MORPHOLOGYVISUALIZATION_API MaximaEdgePairs : public Processor { 
public:
    MaximaEdgePairs();
    virtual ~MaximaEdgePairs() = default;
     
    virtual void process() override;

    virtual const ProcessorInfo getProcessorInfo() const override;
    static const ProcessorInfo processorInfo_;

private:
    DataInport<VectorSpringMassTrajectory> trajectoryMassSpringPort_;
    MeshOutport meshMaximaOut_;
    DataOutport<std::vector<vec2>> allEdgePairs_;

    //Maxima representation property
    BoolProperty showAllMaximas_;
    FloatProperty meshScale_;
    FloatProperty valenceRadius_;
    FloatProperty plotMeshRadius_;
    TransferFunctionProperty tf_;
    OptionPropertyInt logVerbosity_;
};

} // namespace

#endif // IVW_MAXIMAEDGEPAIRS_H

