/*********************************************************************************
 *
 * Inviwo - Interactive Visualization Workshop
 *
 * Copyright (c) 2016-2019 Inviwo Foundation
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *********************************************************************************/

#include <modules/morphologyvisualization/processors/meshcontainers.h>

namespace inviwo {

// The Class Identifier has to be globally unique. Use a reverse DNS naming scheme
const ProcessorInfo MeshContainers::processorInfo_{
    "org.inviwo.MeshContainers",      // Class identifier
    "MeshContainers",                // Display name
    "Data Creation",              // Category
    CodeState::Experimental,  // Code state
    Tags::None,               // Tags
};
const ProcessorInfo MeshContainers::getProcessorInfo() const {
    return processorInfo_;
}

MeshContainers::MeshContainers()
    : Processor()
    , mesh_("mesh")
	, positionContainer_("positionContainer")
	, colorContainer_("colorContainer") 
	, convertToWorldSpace_("convertToWorldSpace","Convert To World Space", true)
{
    
	addPort(mesh_);
	addPort(positionContainer_);
	addPort(colorContainer_);

	addProperty(convertToWorldSpace_);
}
    
void MeshContainers::process() {
	auto geom = mesh_.getData();

	auto posBuffer =
		dynamic_cast<const Vec3BufferRAM*>(geom->getBuffer(0)->getRepresentation<BufferRAM>());
	if (posBuffer == nullptr) return;

	

	auto colBuffer =
		dynamic_cast<const Vec4BufferRAM*>(geom->getBuffer(2)->getRepresentation<BufferRAM>());
	if (colBuffer == nullptr) return;

	auto posContainer = posBuffer->getDataContainer();
	auto colContainer = colBuffer->getDataContainer();
	auto toWorld = geom->getModelMatrix();

	auto p = std::make_shared<std::vector<vec3>>(posContainer.begin(), posContainer.end());
	auto c = std::make_shared<std::vector<vec4>>(colContainer.begin(), colContainer.end());

	if (convertToWorldSpace_) {
		std::for_each(p->begin(), p->end(), [&](auto &pos) { 
			auto pw = toWorld * vec4(pos, 1);
			pos = vec3(pw) / pw.w;
		});
	}


	positionContainer_.setData(p);
	colorContainer_.setData(c);
}

} // namespace

