/*********************************************************************************
 *
 * Inviwo - Interactive Visualization Workshop
 *
 * Copyright (c) 2017-2019 Inviwo Foundation
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 *********************************************************************************/

#ifndef IVW_GRAPHTOCIRCULARPLOTMESH_H
#define IVW_GRAPHTOCIRCULARPLOTMESH_H

#include <modules/morphologyvisualization/morphologyvisualizationmoduledefine.h>
#include <inviwo/core/common/inviwo.h>
#include <inviwo/core/processors/processor.h>
#include <inviwo/core/properties/ordinalproperty.h>
#include <modules/networkanalysis/ports/graphport.h>
#include <modules/networkanalysis/utils/networkgraphutils.h>
#include <modules/networkanalysis/properties/networkgraphlayoutproperty.h>
#include <inviwo/core/properties/directoryproperty.h>
#include <inviwo/core/properties/fileproperty.h>
#include <modules/networkanalysis/ports/graphport.h>
#include <inviwo/core/properties/transferfunctionproperty.h>


namespace inviwo {

class IVW_MODULE_MORPHOLOGYVISUALIZATION_API GraphToCircularPlotMesh : public Processor {
public:
    GraphToCircularPlotMesh();
    virtual ~GraphToCircularPlotMesh() = default;
     
    virtual void process() override;
    void clearAllMesh();
    virtual const ProcessorInfo getProcessorInfo() const override;
    static const ProcessorInfo processorInfo_;
private:
    std::shared_ptr<BasicMesh> constructLevelsOfNetworks(NetworkGraphInfoConstSequence& networkSequence);
    std::shared_ptr<BasicMesh> constructCircularNodesAndEdgesOfNetworks(NetworkGraphInfoConstSequence& networkSequence, DrawType geometryType, bool ovverrideColor);
    

    GraphInfoMultiInport inport_;
    MeshOutport networkLevelsMeshOut_;
    MeshOutport meshNetworkGraphPathNodesOut_;
    MeshOutport meshNetworkGraphPathEdgesOut_;
    MeshOutport meshNetworkGraphPathUnifiedEdgesOut_;
    BoolProperty equiDistance_;
    BoolProperty sortPlanarNodes_;
    BoolProperty fitMeshBound_;
    FloatProperty radialOffset_;
    FloatProperty radialScale_;
    TransferFunctionProperty tf_;
    TransferFunctionProperty tfEdgeColor_;
    IntProperty selectedNetworkIndex_;

    std::shared_ptr<BasicMesh> levelGraphMesh_;
    std::shared_ptr<BasicMesh> nodeGraphMesh_;
    std::shared_ptr<BasicMesh> edgeGraphMesh_;
    std::shared_ptr<BasicMesh> unifiedEdgeGraphMesh_;

    NetworkGraphInfoConstSequence unifiedNetworkGraphSequence_;
    //NetworkGraphLayoutProperty networkGraphLayoutProperty_;
};

} // namespace

#endif // IVW_GRAPHTOCIRCULARPLOTMESH_H

