/*********************************************************************************
 *
 * Inviwo - Interactive Visualization Workshop
 *
 * Copyright (c) 2016-2019 Inviwo Foundation
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *********************************************************************************/

#ifndef IVW_SKELETONEXTRACTION_H
#define IVW_SKELETONEXTRACTION_H

#include <modules/morphologyvisualization/morphologyvisualizationmoduledefine.h>
#include <inviwo/core/common/inviwo.h>
#include <inviwo/core/processors/processor.h>
#include <inviwo/core/properties/ordinalproperty.h>
#include <inviwo/core/ports/meshport.h>
#include <inviwo/core/ports/volumeport.h>
#include <inviwo/core/datastructures/geometry/basicmesh.h>
#include <inviwo/core/properties/compositeproperty.h>
#include <inviwo/core/properties/boolproperty.h>
#include <modules/morphologyvisualization/datastructures/skeletoninfo.h>


namespace inviwo {

/** \docpage{org.inviwo.SkeletonExtraction, SkeletonExtraction}
 * ![](org.inviwo.SkeletonExtraction.png?classIdentifier=org.inviwo.skeletonextraction)
 * Explanation of how to use the processor.
 *
 * ### Inports
 *   * __<Inport1>__ <description>.
 *
 * ### Outports
 *   * __<Outport1>__ <description>.
 *
 * ### Properties
 *   * __<Prop1>__ <description>.
 *   * __<Prop2>__ <description>
 */

/**
 * \class SkeletonExtraction
 * \brief Extract skeleton from binary volume
 * This uses Distance-Ordered Homotopic Thinning ( Chris Pudney, 1998)
 */
class IVW_MODULE_MORPHOLOGYVISUALIZATION_API SkeletonExtraction : public Processor {
public:
    SkeletonExtraction();
    virtual ~SkeletonExtraction() = default;
    virtual void process() override;
    virtual const ProcessorInfo getProcessorInfo() const override;
    static const ProcessorInfo processorInfo_;

    enum ExtractionMode {
        ThresholdFilterValue = 1,
        DOHThinning = 2,
    };

	enum ContextVoxelVisibility {
		ForeGroundOnly = 1 << 0,
		BackGround = 1 << 1,
		DistanceTransform = 1 << 2,
		Segments = 1 << 3
	};    

	

private:
    VolumeInport binaryVolume_;
    VolumeInport distanceVolume_;
    MeshOutport skeletalPointsMeshOutport_;

    FloatProperty distanceThreshold_;
    OptionPropertyInt extractionMethod_;

	CompositeProperty visibilityProperties_;	
	BoolProperty showExteriors_;
	BoolProperty showEndPoints_;
	BoolProperty showMaximalPoints_;
	BoolProperty showBackGround_;
	BoolProperty showSkeleton_;

	CompositeProperty context_;
	OptionPropertyInt contextOptions_;

	IntProperty itreations_;

	//BoolProperty showForeGround_;
	//BoolProperty showSegments_;
	//BoolProperty showDistanceTransform_;

    std::vector<std::shared_ptr<BasicMesh>> skeletalMeshes_;

	//convenient stuff refactored
	void extractSkeleton();
	std::shared_ptr<BasicMesh> generateVoxelMesh(const skeletonutil::VoxelMap& voxelMap);
	void fillupVoxelInfo(skeletonutil::VoxelMap& allVoxels, skeletonutil::VoxelMap& foregroundVoxels, skeletonutil::VoxelMap& backgroundVoxels);
	std::shared_ptr<skeletonutil::SkeletonInfo> ski_;
};

}  // namespace

#endif  // IVW_SKELETONEXTRACTION_H
