/*********************************************************************************
 *
 * Inviwo - Interactive Visualization Workshop
 *
 * Copyright (c) 2017-2019 Inviwo Foundation
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 *********************************************************************************/

#ifndef IVW_VARIABILITYPLOTUTIL_H
#define IVW_VARIABILITYPLOTUTIL_H

#include <modules/morphologyvisualization/morphologyvisualizationmoduledefine.h>
#include <inviwo/core/common/inviwo.h>

#pragma  optimize ("", off)


namespace inviwo {

namespace variabilityplotutil {
//
//template<class It, class Compare>
//bool next_combination(It first, It mid, It last, Compare comp) {
//    std::sort(mid, last, std::tr1::bind(comp, std::tr1::placeholders::_2
//        , std::tr1::placeholders::_1));
//    return std::next_permutation(first, last, comp);
//}
//
//template<class datatype>
//std::vector<datatype> combination(std::vector<datatype> combo, int combinationsToGenerate=2) {
//    std::vector<datatype> output;
//    std::map<std::vector<datatype>, int> removeDuplicateMap;
//
//    while (next_combination(combo.begin(), combo.begin() + combinationsToGenerate, combo.end(), std::less<datatype>())) {
//
//        std::vector<datatype> c;
//
//        for (std::vector<datatype>::iterator iter = combo.begin(); iter != combo.end(); ++iter)
//            c.push_back(*iter);
//
//        removeDuplicateMap[c] += 1;
//    };
//
//    for (auto& m : removeDuplicateMap) {
//        output.insert(output.end(), m.first.begin(), m.first.end());
//    }
//
//    return output;
//}


// http://stackoverflow.com/questions/2211915/combination-and-permutation-in-c/2616837#2616837
template <typename Iterator>
inline bool next_combination(const Iterator first, Iterator k, const Iterator last)
{
    if ((first == last) || (first == k) || (last == k))
        return false;
    Iterator itr1 = first;
    Iterator itr2 = last;
    ++itr1;
    if (last == itr1)
        return false;
    itr1 = last;
    --itr1;
    itr1 = k;
    --itr2;
    while (first != itr1)
    {
        if (*--itr1 < *itr2)
        {
            Iterator j = k;
            while (!(*itr1 < *j)) ++j;
            std::iter_swap(itr1, j);
            ++itr1;
            ++j;
            itr2 = k;
            std::rotate(itr1, j, last);
            while (last != j)
            {
                ++j;
                ++itr2;
            }
            std::rotate(k, itr2, last);
            return true;
        }
    }
    std::rotate(first, k, last);
    return false;
}

template<class datatype>
std::vector<datatype> combination(std::vector<datatype> combo, int k=2) {
    std::vector<datatype> output;
    std::map<std::vector<datatype>, int> removeDuplicateMap;

    std::vector<datatype> startc(combo.begin(), combo.begin() + k);
    removeDuplicateMap[startc] += 1;

    while (next_combination(combo.begin(), combo.begin() + k, combo.end())) {

        std::vector<datatype> c;

        for (std::vector<datatype>::iterator iter = combo.begin(); iter != combo.begin() + k; ++iter)
            c.push_back(*iter);

        removeDuplicateMap[c] += 1;
    };

    for (auto& m : removeDuplicateMap) {
        output.insert(output.end(), m.first.begin(), m.first.end());
    }

    return output;
}


/*
float bandDepth2(const std::vector<vec3>& curve1,
    const std::vector<vec3>& curve2,
    const std::vector<vec3>& curve) {
    float depth = 0.0f;

    return depth;
}

float modifiedBandDepth(const std::vector<vec3>& curve1,
    const std::vector<vec3>& curve2,
    const std::vector<vec3>& curve) {
    float depth = 0.0f;

    return depth;
}
*/
} //name space variability stuff

} // namespace

#endif // IVW_VARIABILITYPLOTUTIL_H

