/*********************************************************************************
 *
 * Inviwo - Interactive Visualization Workshop
 *
 * Copyright (c) 2016-2019 Inviwo Foundation
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *********************************************************************************/

#include <warn/push>
#include <warn/ignore/all>
#include <gtest/gtest.h>
#include <warn/pop>

#include <inviwo/core/common/inviwo.h>
#include <modules/morphologyvisualization/util/ridgeassignment.h>

namespace inviwo {

TEST(MorphologyRidgeAssignment, Empty) {
    const std::vector<int> maximas = {};
    const std::vector<std::pair<int, int>> ridges = {};
    auto res = morphviz::assignRidgesFromMaximas(maximas.cbegin(), maximas.cend());
    ASSERT_EQ(ridges, res);
}

TEST(MorphologyRidgeAssignment, One) {
    const std::vector<int> maximas = {0};
    const std::vector<std::pair<int, int>> ridges = {};
    auto res = morphviz::assignRidgesFromMaximas(maximas.cbegin(), maximas.cend());
    ASSERT_EQ(ridges, res);
}

TEST(MorphologyRidgeAssignment, SameTwo) {
    const std::vector<int> maximas = {0, 0};
    const std::vector<std::pair<int, int>> ridges = {{0, 0}, {0, 0}};
    auto res = morphviz::assignRidgesFromMaximas(maximas.cbegin(), maximas.cend());
    ASSERT_EQ(ridges, res);
}

TEST(MorphologyRidgeAssignment, DifferentTwoA) {
    const std::vector<int> maximas = {0, 1};
    const std::vector<std::pair<int, int>> ridges = {{0, 1}, {0, 1}};
    auto res = morphviz::assignRidgesFromMaximas(maximas.cbegin(), maximas.cend());
    ASSERT_EQ(ridges, res);
}
TEST(MorphologyRidgeAssignment, DifferentTwoB) {
    const std::vector<int> maximas = {1, 0};
    const std::vector<std::pair<int, int>> ridges = {{0, 1}, {0, 1}};
    auto res = morphviz::assignRidgesFromMaximas(maximas.cbegin(), maximas.cend());
    ASSERT_EQ(ridges, res);
}

TEST(MorphologyRidgeAssignment, TwoMaximas) {
    const std::vector<int> maximas = {0, 0, 0, 1, 1, 1};
    const std::vector<std::pair<int, int>> ridges = {{0, 1}, {0, 1}, {0, 1},
                                                     {0, 1}, {0, 1}, {0, 1}};
    auto res = morphviz::assignRidgesFromMaximas(maximas.cbegin(), maximas.cend());
    ASSERT_EQ(ridges, res);
}

TEST(MorphologyRidgeAssignment, ThreeMaximas) {
    const std::vector<int> maximas = {0, 0, 0, 1, 1, 1, 1, 2, 2, 2};
    const std::vector<std::pair<int, int>> ridges = {{0, 1}, {0, 1}, {0, 1}, {0, 1}, {0, 1},
                                                     {1, 2}, {1, 2}, {1, 2}, {1, 2}, {1, 2}};
    auto res = morphviz::assignRidgesFromMaximas(maximas.cbegin(), maximas.cend());
    ASSERT_EQ(ridges, res);
}


TEST(MorphologyRidgeAssignment, ThreeMixedMaximasA) {
    const std::vector<int> maximas = { 0, 0, 1, 0, 1, 1, 2, 1, 2, 2 };
    const std::vector<std::pair<int, int>> ridges = { {0, 1}, {0, 1}, {0, 1}, {0, 1}, {0, 1},
                                                     {1, 2}, {1, 2}, {1, 2}, {1, 2}, {1, 2} };
    auto res = morphviz::assignRidgesFromMaximas(maximas.cbegin(), maximas.cend());
    ASSERT_EQ(ridges, res);
}

TEST(MorphologyRidgeAssignment, ThreeMThreeMixedMaximasB) {
    const std::vector<int> maximas = {0, 0, 1, 0, 0, 1, 1, 0, 1, 2, 2, 1, 1, 2, 2, 1, 2, 2};
    const std::vector<std::pair<int, int>> ridges = {
        {0, 1}, {0, 1}, {0, 1}, {0, 1}, {0, 1}, {0, 1}, {0, 1}, {0, 1}, {1, 2},
        {1, 2}, {1, 2}, {1, 2}, {1, 2}, {1, 2}, {1, 2}, {1, 2}, {1, 2}, {1, 2}};
    auto res = morphviz::assignRidgesFromMaximas(maximas.cbegin(), maximas.cend());
    ASSERT_EQ(ridges, res);
}

} // namespace

