/*********************************************************************************
 *
 * Inviwo - Interactive Visualization Workshop
 *
 * Copyright (c) 2016-2019 Inviwo Foundation
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *********************************************************************************/

#include <modules/morphologyvisualization/util/skeletonadjacencymapper.h>
#include <inviwo/core/util/indexmapper.h>
#include <modules/morphologyvisualization/datastructures/skeletonvoxels.h>
#include <modules/base/datastructures/kdtree.h>



namespace inviwo {

const util::IndexMapper3D skeletonutil::NAdjacencyMapper::mapper3x3_ = util::IndexMapper3D(size3_t(3));

namespace skeletonutil {

NAdjacencyMapper::NAdjacencyMapper() {
    ivec3 c(1);

    n6i_ = getN6(c);
    n26i_ = getN26(c);
    n27i_ = getN27(c);
    n18i_ = getN18(c);
}

/*
inline int LabelComponent(unsigned short width, unsigned short height,
unsigned char* input, int* output, int labelNo, const ivec3& pos, std::vector<size3_t> neighbours)	{
auto index =
if (input[index] == 0) return;   /* This pixel is not part of a component * /
if (output[index] != 0) return;   /* This pixel has already been labelled  * /
output[index] = labelNo;

/* Now label the 4 neighbours: * /
if (x > 0)        LabelComponent(width, height, input, output, labelNo, x - 1, y);   /* left  pixel * /
if (x < width - 1)  LabelComponent(width, height, input, output, labelNo, x + 1, y);   /* right pixel * /
if (y > 0)        LabelComponent(width, height, input, output, labelNo, x, y - 1);   /* upper pixel * /
if (y < height - 1) LabelComponent(width, height, input, output, labelNo, x, y + 1);   /* lower pixel * /
}
*/

inline int NAdjacencyMapper::LabelSerially(const VoxelMap& input, const int labelNo, const ivec3& pos,
	const std::vector<ivec3>& neighbourOffsets, VoxelMap& output,
	int& usedLabels,	
	size_t splitSize) {

	//Avoid full recursion
	//TODO: Replace with kd-tree
	
	//split output 
	ivec3 p = pos;
	VoxelMap output_segment;
	std::vector<ivec3> lastVisited;

	auto& boundaryTest = output.begin()->second.boundaryTest_;
	auto& mapping = output.begin()->second.mapping_;
	auto& getNeighbours = output.begin()->second.getNeighbours_;

	//here use kdtree and add neighbours
	//auto depth = 0;
	//#define MAX_RECURSION 1000
	//while (depth<1) {
		
		//work on a small segment to avoid deep recursion
		/*size_t segSize = 0;
		for (auto& oitr: output) {
			if (oitr.second.labels_ == 0) {
				output_segment[oitr.first] = oitr.second;
				segSize++;
				p = oitr.second.pos_;
				if (segSize >= splitSize) break;
			}
		}

		if (segSize == 0) break;*/

		output_segment[mapping(p)] = output[mapping(p)];
	
		//label recursively. work on a small segment
		//lastVisited.clear();
		//auto prevSeg = output_segment[mapping(p)].labels_;
		output_segment[mapping(p)].labels_ = 0; //this is allow restart from p

		auto recurDepthCount = 0;
		LabelRecusively(input, labelNo, p, neighbourOffsets, output_segment, usedLabels, lastVisited, recurDepthCount);
		output_segment[mapping(p)].labels_ = labelNo;

		for (auto &os : output_segment)
			output[os.first] = os.second;

		//collect all the exit points
		while (lastVisited.size()) {

			std::vector<ivec3> removeVisited;
			//copy set of exit points and re-run labeling again
			for (auto lp : lastVisited) {
				output_segment[mapping(lp)] = output[mapping(lp)];
				//add octree neighbourhood
				auto neigbourIndcies = getNeighbours(lp, 5.0);
				for (auto& ngh : neigbourIndcies)
					output_segment[mapping(ngh)] = output[mapping(ngh)];
				if (output_segment[mapping(lp)].labels_ != 0)
					removeVisited.push_back(lp);
			}

			if (removeVisited.size()) {
				for (auto& lv : removeVisited) {
					auto f = std::find(lastVisited.begin(), lastVisited.end(), lv);
					if (f!=lastVisited.end())
						lastVisited.erase(f);
				}
			}

			if (!lastVisited.size()) break;
			
			//start again from p
			p = lastVisited.back();
			lastVisited.erase(lastVisited.end()-1);

			//lastVisited.clear();

			//
			//auto prevLabel = output_segment[mapping(p)].labels_;
			output_segment[mapping(p)].labels_ = 0; //this is allow restart from p

			recurDepthCount = 0;
			LabelRecusively(input, labelNo, p, neighbourOffsets, output_segment, usedLabels, lastVisited, recurDepthCount);
			output_segment[mapping(p)].labels_ = labelNo;

			for (auto &os : output_segment)
				output[os.first] = os.second;


			/*for (auto& lv : localVisited) {
				auto f = std::find(lastVisited.begin(), lastVisited.end(), lv);
				if (f == lastVisited.end())
					lastVisited.push_back(lv);
			}*/

		};

		for (auto &os : output_segment)
			output[os.first] = os.second;

	//	depth++;
	//};

	//if (depth >= MAX_RECURSION) {
	//	//LogWarn("Incomplete Operation and forced exit.");
	//	std::cout << "Incomplete Operation and forced exit. Pos("  << pos << ")" << std::endl;
	//}
	

	return usedLabels;
}

int NAdjacencyMapper::LabelRecusively(const VoxelMap& input, const int labelNo, const ivec3& pos,
                           const std::vector<ivec3>& neighbourOffsets, VoxelMap& output,
                           int& usedLabels, 
							std::vector<ivec3>& lastVisited,
						   int& recursiondepth) {
	
	auto& boundaryTest = output.begin()->second.boundaryTest_;
	auto& mapping = output.begin()->second.mapping_;

	

	auto stopCriteria = [&] (const ivec3& spos) {
		
		if (!boundaryTest(spos)) {
			return false;
		}

		auto sindex = mapping(spos);

		if (input.find(sindex) == input.end()) return false;

		if (input.find(sindex)->second.hasVoxelFlag(VoxelFlags::BackGround))
			return false;

		if (output.find(sindex) == output.end()) {
			auto lvit = std::find(lastVisited.begin(), lastVisited.end(), spos);
			if (lvit == lastVisited.end())
				lastVisited.push_back(spos); //restart from here
			return false;
		}

		if (output[sindex].labels_ != 0) return false;

		//if (lastVisited.size() > 500) return false;

		if ( recursiondepth >= 1000 ) {
			auto lvit = std::find(lastVisited.begin(), lastVisited.end(), spos);
			if (lvit == lastVisited.end())
				lastVisited.push_back(spos); //restart from here
			return false;
		}

		return true;
	};

	
	if (!stopCriteria(pos))
		return usedLabels;
	
	auto index = mapping(pos);

    output[index].labels_ = labelNo;
    usedLabels++;
	recursiondepth++;
	

    for (auto& n : neighbourOffsets) {
        ivec3 newPos = ivec3(pos) + ivec3(n);
		
        if (pos == newPos) continue;

		//avoid recursion call if output already set
		
		if (!stopCriteria(newPos))
			continue;

        LabelRecusively(input, labelNo, newPos, neighbourOffsets, output, usedLabels, lastVisited, recursiondepth);
    }

    return usedLabels;
}

inline int NAdjacencyMapper::countComponents(const Voxel& v, const std::vector<ivec3>& neighbourOffsets,
                           bool flipInputImage) {
    // Do recursive labeling
    VoxelMap input;

	std::function<bool(size3_t)> boundaryTest = [&](size3_t newPos) {
		if (newPos.x < 0 || newPos.y < 0 || newPos.z < 0 || newPos.x > 2 || newPos.y > 2 ||
			newPos.z > 2) {
			return false;
		}
		return true;
	};

	std::function<size_t(size3_t)> mapping = [&](ivec3 newPos) {
		return mapper3x3_(newPos);
	};

	for (auto& n : n27i_) {
		Voxel inv;
		inv.setVoxelFlag(VoxelFlags::None);
		inv.labels_ = 0;
		inv.boundaryTest_ = boundaryTest;
		inv.mapping_ = mapping;
		input[mapper3x3_(n)] = inv;
	}

    // We use (m,n) = (26,6)
    for (auto& n : n27i_) {
        auto p = v.hasNeighbour(n);
        if (flipInputImage) p = !p;  // used for labeling on background images
        if (n == ivec3(1)) p = false;    // remove center pixel from input
        if (p==0)
			input[mapper3x3_(n)].setVoxelFlag(VoxelFlags::BackGround);
		else
			input[mapper3x3_(n)].setVoxelFlag(VoxelFlags::ForeGround);
    }

	VoxelMap output;
	for (auto& n : n27i_) {
		auto i = mapper3x3_(n);
		output[i] = input[i];
		output[i].boundaryTest_ = boundaryTest;
		output[i].mapping_ = mapping;
	}

	

    auto labelNo = 2;
    for (auto& n : n27i_) {
        if (n == ivec3(1)) continue;

        //for (size_t j = 0; j < input.size(); j++) output[j] = 0;

        auto newPos = n;
        
        auto usedLabels = 0;
		auto recurDepthCount = 0;
		std::vector<ivec3> lastVisited; // (newPos);
        LabelRecusively(input, labelNo, newPos, neighbourOffsets, output, usedLabels, lastVisited, recurDepthCount);

        if (usedLabels) labelNo++;
    }

    return labelNo-2;
}

inline int NAdjacencyMapper::countComponents_N6(const Voxel& v, bool flipInputImage) {
    auto neighbours = getN6(ivec3(0));
    return countComponents(v, neighbours, flipInputImage);
}

inline int NAdjacencyMapper::countComponents_N26(const Voxel& v, bool flipInputImage) {
    auto neighbours = getN26(ivec3(0));
    return countComponents(v, neighbours);
}

// always returns foreground counts in the neighbourhood
inline int NAdjacencyMapper::countN_6(const Voxel& v) {
    int count = 0;
    for (auto& n : n6i_) {
        if (v.hasNeighbour(n)) count++;
    }
    return count;
}

inline int NAdjacencyMapper::countN_26(const Voxel& v) {
    int count = 0;
    for (auto& n : n26i_)
        if (v.hasNeighbour(n)) count++;
    return count;
}

inline int NAdjacencyMapper::countN_18(const Voxel& v) {
    int count = 0;
    for (auto& n : n18i_)
        if (v.hasNeighbour(n)) count++;
    return count;
}

inline bool NAdjacencyMapper::isExterior(const Voxel& v) {
    // We use (m,n) = (26,6)
    // Exterior Definition:
    // points n-adjacent to the background are called exterior points;
    // otherwise they are called interior points.
    auto background = 6 - countN_6(v);
    if (background == 0) {
        // std::cout << "Exterior Info :" << v.pos_ << "Flags :" << v.neighbours_ << "BG : " <<
        // background << std::endl;
        return false;
    }
    return true;
}

inline bool NAdjacencyMapper::isEndPoint(const Voxel& v) {
    auto foreground = countN_26(v);
    return (foreground < 2);
}

inline bool NAdjacencyMapper::isDangling(const Voxel& v) {
    auto foreground = countN_26(v);
    return (foreground == 0);
}

inline bool NAdjacencyMapper::isCenterOfMaximalSphere(const Voxel& v) { 
	//this in order to be efficient need to be computed globally using getMaximalCenters()
	if (v.hasVoxelFlag(VoxelFlags::Maximum)) return true;
	return false;
	return isCenterOfMaximalSphere(v);
	
}

inline bool NAdjacencyMapper::isSimple(const Voxel& v) {
    // We use (m,n) = (26,6)

    // p is m - adjacent to just one connected component of N*(p) (n) F
    auto c1 = countComponents_N26(v);

    // p is n - adjacent to just one connected component of N*(p) - F
    auto c2 = countComponents_N6(v, true);

    return (c1 <= 1) && (c2 <= 1);
}

bool NAdjacencyMapper::isDeletable(const Voxel& v) {
    //  isSimple() - Rule A
    //  isEndPointPointOfMedialEdge() - Rule A
    //  isCenterOfMaximalSphere() - Rule B
    if (isSimple(v)) {
        if (!isEndPoint(v) && !isCenterOfMaximalSphere(v)) {
            return true;
        }
    }

    return false;
}


std::vector<ivec3> NAdjacencyMapper::getN6(ivec3 c) {
	std::vector<ivec3> n6;
	ivec3 x(1, 0, 0);
	ivec3 y(0, 1, 0);
	ivec3 z(0, 0, 1);

	n6 = { (c - x), (c + x), (c - y), (c + y), (c - z), (c + z) };

	return n6;
}

std::vector<ivec3> NAdjacencyMapper::getN26(ivec3 c) {
	std::vector<ivec3> n26;
	for (int x = c.x - 1; x <= c.x + 1; x++) {
		for (int y = c.y - 1; y <= c.y + 1; y++) {
			for (int z = c.z - 1; z <= c.z + 1; z++) {
				if (x == c.x && y == c.y && z == c.z) {
					// center
					continue;
				}
				n26.push_back(ivec3(x, y, z));
			}
		}
	}

	return n26;
}

std::vector<ivec3> NAdjacencyMapper::getN27(ivec3 c) {
	std::vector<ivec3> n27;
	for (int x = c.x - 1; x <= c.x + 1; x++) {
		for (int y = c.y - 1; y <= c.y + 1; y++) {
			for (int z = c.z - 1; z <= c.z + 1; z++) {
				n27.push_back(ivec3(x, y, z));
			}
		}
	}

	return n27;
}

std::vector<ivec3> NAdjacencyMapper::getN18(ivec3 c) {
	// TODO: Rewrite this to account for c
	std::vector<ivec3> n18;

	for (int x = c.x - 1; x <= c.x + 1; x++) {
		for (int y = c.y - 1; y <= c.y + 1; y++) {
			for (int z = c.z - 1; z <= c.z + 1; z++) {
				if (x == 0 && y == 0 && z == 0) continue;
				if (x == 0 && y == 0 && z == 2) continue;

				if (x == 2 && y == 0 && z == 0) continue;
				if (x == 2 && y == 0 && z == 2) continue;

				if (x == 0 && y == 2 && z == 0) continue;
				if (x == 0 && y == 2 && z == 2) continue;

				if (x == 2 && y == 2 && z == 0) continue;
				if (x == 2 && y == 2 && z == 2) continue;

				if (x == 1 && y == 1 && z == 1) continue;

				n18.push_back(ivec3(x, y, z));
			}
		}
	}

	return n18;
}


//function to gather 
//std::function<void(const VoxelMap& srcVoxelMap, VoxelMap& dstVoxelMap, VoxelFlags flag)> getFlagedVoxels = [&](const VoxelMap& srcVoxelMap, VoxelMap& dstVoxelMap, VoxelFlags flag) {
inline void getFlagedVoxels(const VoxelMap& srcVoxelMap, VoxelMap& dstVoxelMap, VoxelFlags flag) {

	for (auto& ov : srcVoxelMap) {
		Voxel sv = ov.second;
		Voxel v = ov.second;
		auto cpos = v.pos_;

		v.unsetVoxelFlag(flag);

		if (sv.hasVoxelFlag(flag)) {
			v.setVoxelFlag(flag);
			dstVoxelMap[v.index_] = v;
		}

		/*

		if (flag & VoxelFlags::ForeGround) {

			if (sv.hasVoxelFlag(VoxelFlags::ForeGround)) {
				v.setVoxelFlag(VoxelFlags::ForeGround);
				dstVoxelMap[v.index_] = v;
			}
		}

		if (flag & VoxelFlags::BackGround) {
			if (sv.hasVoxelFlag(VoxelFlags::BackGround)) {
				v.setVoxelFlag(VoxelFlags::BackGround);
				dstVoxelMap[v.index_] = v;
			}
		}
		*/

	}
}

//neighbour info for given voxel
inline void setNeighbourInfo(VoxelMap& srcVoxelMap, Voxel& v, size3_t dim) {
	ivec3 rdim(dim);
	rdim += ivec3(-1);
	util::IndexMapper3D index((dim));

	auto cpos = v.pos_;

	//Voxel v;
	//v = ov.second;

	for (int x = 0; x < 3; x++)
		for (int y = 0; y < 3; y++)
			for (int z = 0; z < 3; z++)
				v.unsetNeighbour(ivec3(x, y, z));

	ivec3 p_start = cpos;
	p_start += ivec3(-1);

	ivec3 p_end = cpos;
	p_end += ivec3(1);


	for (int x = p_start.x; x <= p_end.x; x++) {
		for (int y = p_start.y; y <= p_end.y; y++) {
			for (int z = p_start.z; z <= p_end.z; z++) {

				ivec3 id(x, y, z);
				auto cid = glm::clamp(id, ivec3(0), rdim);
				auto cellid = id - p_start;

				if (id.x > rdim.x || id.y > rdim.y || id.z > rdim.z ||
					id.x < 0 || id.y < 0 || id.z < 0) {
					v.unsetNeighbour(cellid);
					continue;
				}

				if (srcVoxelMap.find(index(cid)) != srcVoxelMap.end()) {

					auto dv = srcVoxelMap[index(cid)];

					if (dv.hasVoxelFlag(VoxelFlags::ForeGround)) {
						v.setNeighbour(cellid);
					}
					else {
						v.unsetNeighbour(cellid);
					}
				}
				else v.unsetNeighbour(cellid);


			}
		}
	}
}

inline dvec2 computeDistanceTransform(const VoxelMap& srcVoxelMap, VoxelMap& dstVoxelMap) {

	K3DTree<Voxel, float> tree;

	for (auto&sv : srcVoxelMap)
		tree.insert(sv.second.pos_, sv.second);

	auto maxRadius = 25.0;
	auto maxDist = 0.0;
	auto minDist = 0.0;

	for (auto&dv : dstVoxelMap) {
		auto nghs = tree.findCloseTo(dv.second.pos_, maxRadius);
		if (!nghs.size())
			continue;

		Voxel closestVoxel = nghs.front()->get();

		for (auto& n : nghs) {
			auto p = n->get();
			auto d = glm::distance(p.pos_, dv.second.pos_);

			if (d <= glm::distance(closestVoxel.pos_, dv.second.pos_))
				closestVoxel = p;			
		}

		dv.second.nPos_ = closestVoxel.pos_;
		dv.second.distance_ = glm::distance(closestVoxel.pos_, dv.second.pos_);
		if (dv.second.distance_ > maxDist) maxDist = dv.second.distance_;
		if (dv.second.distance_ < minDist) minDist = dv.second.distance_;
	}

	return dvec2(minDist, maxDist);
}

}  // namespace

}  // namespace
