/*********************************************************************************
 *
 * Inviwo - Interactive Visualization Workshop
 *
 * Copyright (c) 2016-2019 Inviwo Foundation
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *********************************************************************************/

#ifndef IVW_SKELETONADJACENCYMAPPER_H
#define IVW_SKELETONADJACENCYMAPPER_H

#include <modules/morphologyvisualization/morphologyvisualizationmoduledefine.h>
#include <inviwo/core/common/inviwo.h>
#include <modules/morphologyvisualization/datastructures/skeletonvoxels.h>

namespace inviwo {

namespace skeletonutil {
// Distance - Ordered Homotopic Thinning : A Skeletonization Algorithm for 3D Digital Images

struct IVW_MODULE_MORPHOLOGYVISUALIZATION_API NAdjacencyMapper {

    std::vector<ivec3> n6i_;
    std::vector<ivec3> n18i_;
    std::vector<ivec3> n26i_;
    std::vector<ivec3> n27i_;
    static const util::IndexMapper3D mapper3x3_;

    NAdjacencyMapper();
    int LabelRecusively(const VoxelMap& input, const int labelNo, const ivec3& pos,
                        const std::vector<ivec3>& neighbourOffsets, VoxelMap& output,
                        int& usedLabels, std::vector<ivec3>& lastVisited, int &recursionDepth);

    inline int LabelSerially(const VoxelMap& input, const int labelNo, const ivec3& pos,
                             const std::vector<ivec3>& neighbourOffsets, VoxelMap& output,
                             int& usedLabels,
                             size_t splitSize = 500);

    inline int countComponents(const Voxel& v, const std::vector<ivec3>& neighbourOffsets,
                               bool flipInputImage = false);

    inline int countComponents_N6(const Voxel& v, bool flipInputImage = false);

    inline int countComponents_N26(const Voxel& v, bool flipInputImage = false);

    // always returns foreground counts in the neighbourhood
    inline int countN_6(const Voxel& v);

    inline int countN_26(const Voxel& v);

    inline int countN_18(const Voxel& v);

    inline bool isExterior(const Voxel& v);

    inline bool isEndPoint(const Voxel& v);

    inline bool isDangling(const Voxel& v);

    inline bool isCenterOfMaximalSphere(const Voxel& v);

    inline bool isSimple(const Voxel& v);

    bool isDeletable(const Voxel& v);

    std::vector<ivec3> getN6(ivec3 c);
    std::vector<ivec3> getN26(ivec3 c);
    std::vector<ivec3> getN27(ivec3 c);
    std::vector<ivec3> getN18(ivec3 c);
};

inline void IVW_MODULE_MORPHOLOGYVISUALIZATION_API getFlagedVoxels(const VoxelMap& srcVoxelMap, VoxelMap& dstVoxelMap, VoxelFlags flag);

inline void IVW_MODULE_MORPHOLOGYVISUALIZATION_API setNeighbourInfo(VoxelMap& srcVoxelMap, Voxel& v, size3_t dim);

inline dvec2 IVW_MODULE_MORPHOLOGYVISUALIZATION_API computeDistanceTransform(const VoxelMap& srcVoxelMap, VoxelMap& dstVoxelMap);


}

}  // namespace

#endif  // IVW_SKELETONADJACENCYMAPPER_H
